#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

# use strict;
require Exporter;
require AutoLoader;

use CGI;

$W3C::Util::W3CDebugCGI::REVISION = '$Id: W3CDebugCGI.pm,v 1.24 1999/12/14 13:14:41 eric Exp $ ';

package W3C::Util::W3CDebugCGI;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK @TODO);
@ISA = qw(CGI Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.52;
$DSLI = 'adpO';
@TODO = ('figure out how to use strict with filehandles');

sub new {
    my ($proto, $scriptName, $reUseCGI, $flags) = @_;
    my $class = ref($proto) || $proto;
    $flags = {} if (ref $flags ne 'HASH');
    if ($flags->{'-noStore'}) {
	$self = $class->SUPER::new({});
	bless ($self, $class);
	return $self;
    }
    my $storeIn = $flags->{'-storeIn'};
    my $dieNoOpen = $flags->{'-dieNoOpen'};
    my $logExt = $flags->{'-logExt'};
    $scriptName = 'script.pl' if (!defined $scriptName);
    $scriptName =~ s/([^\/]*\/)*//;
    $scriptName =~ m/([\w\-\.]+)/;
    $scriptName = $1;
    $reUseCGI = 0 if (!defined $reUseCGI);
    $storeIn = '/tmp/' if (!defined $storeIn);
    $storeIn .= '/' if($storeIn !~ /\/\Z/);
    $dieNoOpen = 0 if (!defined $dieNoOpen);
    my ($baseName, $cgiName, $envName, $self);
    #($baseName = $scriptName) =~ s/^.*\/([^\/\.]+).*$/$1/;
    ($baseName = $scriptName) =~ s/^.*\/([^\/]+)$/$1/;
    $cgiName = $storeIn.$baseName.'.cgi';
    $envName = $storeIn.$baseName.'.env';
    if ($reUseCGI) {
#	sysopen(SAVE, $cgiName, O_RDWR, 0666);
	if (open(SAVE, $cgiName)) {
	    $self = $class->SUPER::new(SAVE);
	} else {
	    die "unable to open \"$cgiName\": $!" if ($dieNoOpen);
	    $self = $class->SUPER::new;
	}
	if (open(SAVE, $envName)) {
	    &loadENV(SAVE);
	    close(SAVE);
	} else {
	    die "unable to open \"$envName\": $!" if ($dieNoOpen);
	}
#	if ($ENV{'CONTENT_LENGTH'} > 0) {
#	    open XT, q[3>&1 xterm -title 'Forked Perl debugger' -e sh -c 'tty 1>&3; sleep 10000000' |];
#	    $DB::fork_TTY = <XT>;
#	    chomp $fork_TTY;
#	    $pid = open(STDOUT, "|-");
#	    # continue processing as child of this fork
#	    return if ($pid == 0);
#	    my @out;
#	    foreach my $param ($self->param) {
#		push (@out, $param.'='.$self->param($param));
#	    }
#	    print STDOUT join('&', @out);
#	    exit(0);
#	}
    } else {
#	print "Status: 200\bContent-type: text/plain\n\n<pre>";
#	print "CGI params file: $cgiName\n";
#	print "environment file: $envName\n";
	$self = $class->SUPER::new;
#	sysopen(SAVE, $cgiName, O_RDWR|O_CREAT|O_TRUNC, 0666);
	if (open(SAVE, '>'.$cgiName)) {
	    $self->save(SAVE);
	} else {
	    print "Content-Type: text/plain\n\nunable to open \"$cgiName\": $!";
	    die "unable to open \"$cgiName\": $!" if ($dieNoOpen);
	}
	if (open(SAVE, '>'.$envName)) {
	    &saveENV(SAVE);
	    close(SAVE);
	} else {
	    print "Content-Type: text/plain\n\nunable to open \"$envName\": $!";
	    die "unable to open \"$envName\": $!" if ($dieNoOpen);
	}
	if ($logExt) {
	    require 'sys/syscall.ph';
	    my $timestamp=pack('LL', ());
	    syscall( &SYS_gettimeofday, $timestamp, 0) != -1 or die "gettimeofday: $!";
	    my @timestamp = unpack('LL', $timestamp);
	    $self->{VERSION_ID} = $timestamp = $scriptName.' '.$timestamp[0].'.'.$timestamp[1].' '.`date`;
	    if (open(SAVE, '>>'.$cgiName.$logExt)) {
		print SAVE $timestamp;
		$self->save(SAVE);
	    } else {
		print "Content-Type: text/plain\n\nunable to open \"$cgiName$logExt\": $!";
		die "unable to open \"$cgiName$logExt\": $!" if ($dieNoOpen);
	    }
	    if (open(SAVE, '>>'.$envName.$logExt)) {
		print SAVE $timestamp;
		&saveENV(SAVE);
		close(SAVE);
	    } else {
		die "unable to open \"$envName\": $!" if ($dieNoOpen);
	    }
	}
    }
    if (defined $ENV{'HTTP_AUTHORIZATION'} && !defined $ENV{'REMOTE_USER'}) {
	# apache's in *that* mode
	my ($scheme, $data) = split (' ', $ENV{'HTTP_AUTHORIZATION'});
	
	if ($scheme eq 'Basic') {
	    $data = &decode_base64 ($data);
	    ($ENV{'REMOTE_USER'}, $ENV{'REMOTE_PASSWORD'}) = split(':', $data);
	    # If a script is running in an env where apache is not doing auth, 
	    # the script must check the password itself.
	}
    }
    bless ($self, $class);
#    print $self->header,$self->start_html,$storeIn.$baseName,"<BR>",`pwd`,"<BR>";
    return $self;
}

sub getVersionId {
    my ($self) = @_;
    return $self->{VERSION_ID};
}

sub saveENV {
    my $filehandle = shift;
    $filehandle = &CGI::to_filehandle($filehandle);
    foreach my $key (keys %ENV) {
	my $escKey = &CGI::escape($key);
	my $value = $ENV{$key};
	print $filehandle "$escKey=",&CGI::escape($value),"\n";
    }
    print $filehandle "=\n";    # end of record
}

sub loadENV {
    require "shellwords.pl";
    my $filehandle = shift;
    $filehandle = &CGI::to_filehandle($filehandle);
    %ENV = ();
    foreach my $input (<$filehandle>) {
	chomp($input);
	$input = &CGI::unescape($input);
	my ($key, $value) = split(/=/, $input, 2);
	$ENV{$key} = $value;
    }
}

sub read_multipart {
    my($self,$boundary,$length,$filehandle) = @_;
    my($buffer) = $self->new_MultipartBuffer($boundary,$length,$filehandle);
    return unless $buffer;
    my(%header,$body);
    my $filenumber = 0;
#    print "status: 200\nContent-type: text/plain\n\nmy init!!!\n";
#    print "read_multipart\n";
    while (!$buffer->eof) {
	%header = $buffer->readHeader;
	die "Malformed multipart POST\n" unless %header;

	my($param)= $header{'Content-Disposition'}=~/ name="?([^\";]*)"?/;
#	print "param: \"$param\"\n";

	# Bug:  Netscape doesn't escape quotation marks in file names!!!
	my($filename) = $header{'Content-Disposition'}=~/ filename="?([^\";]*)"?/;
#	print "filename: \"$filename\"\n";

	# add this parameter to our list
	$self->add_parameter($param);

	# If no filename specified, then just read the data and assign it
	# to our parameter list.
	unless ($filename && 0) {
	    my($value) = $buffer->readBody;
	    push(@{$self->{$param}},$value);
	    next;
	}

	my ($tmpfile,$tmp,$filehandle);
      UPLOADS: {
	  # If we get here, then we are dealing with a potentially large
	  # uploaded form.  Save the data to a temporary file, then open
	  # the file for reading.

	  # skip the file if uploads disabled
	  if ($DISABLE_UPLOADS) {
	      while (defined($data = $buffer->read)) { }
	      last UPLOADS;
	  }

	  $tmpfile = new TempFile;
	  $tmp = $tmpfile->as_string;
	  
	  $filehandle = Fh->new($filename,$tmp,$PRIVATE_TEMPFILES);

	  $CGI::DefaultClass->binmode($filehandle) if $CGI::needs_binmode;
	  chmod 0600,$tmp;    # only the owner can tamper with it

	  my ($data);
	  local($\) = '';
	  while (defined($data = $buffer->read)) {
	      print $filehandle $data;
	  }

	  # back up to beginning of file
	  seek($filehandle,0,0);
	  $CGI::DefaultClass->binmode($filehandle) if $CGI::needs_binmode;

	  # Save some information about the uploaded file where we can get
	  # at it later.
	  $self->{'.tmpfiles'}->{$filename}= {
	      name => $tmpfile,
	      info => {%header},
	  };
	  push(@{$self->{$param}},$filehandle);
      }
    }
}

# libwww-perl 5.0.6 base64 decoder:
#
#Copyright 1995, 1996 Gisle Aas.
#
#This library is free software; you can redistribute it and/or
#modify it under the same terms as Perl itself.
#
#Gisle Aas <aas@sn.no>
#
#Based on LWP::Base64 written by Martijn Koster <m.koster@nexor.co.uk>
#and Joerg Reichelt <j.reichelt@nexor.co.uk> and code posted to
#comp.lang.perl <3pd2lp$6gf@wsinti07.win.tue.nl> by Hans Mulder
#<hansm@wsinti07.win.tue.nl>
#
#=cut                      

sub decode_base64 {
    local ($str) = "@_";
    local ($res, $len) = "";
    local($^W) = 0; # unpack("u",...) gives bogus warning in 5.00[123]


    $str =~ tr|A-Za-z0-9+=/||cd;            # remove non-base64 chars
    Carp::croak("Base64 decoder requires string length to be a multiple of 4")
      if length($str) % 4;
    $str =~ s/=+$//;                        # remove padding
    $str =~ tr|A-Za-z0-9+/| -_|;            # convert to uuencoded format
    while ($str =~ /(.{1,60})/gs) {
        $len = chr(32 + length($1)*3/4); # compute length byte
        $res .= unpack("u", $len . $1 );    # uudecode
    }
    return ($res);
}

# convert "http://quake.w3.org/Phonelist.php3" <=> "http://quake.w3.org/Phonelist.php3"
sub escapeName ($) {
    my $toEscape = shift;
    $toEscape = shift if (ref $toEscape);
    my $a = CGI::escape($toEscape);
    $a =~ s/_/_u/g;
    $a =~ s/\%/_p/g;
    return $a;
}

sub unescapeName ($) {
    my $toEscape = shift;
    $toEscape = shift if (ref $toEscape);
    $a = $toEscape;
    $a =~ s/_u/_/g;
    $a =~ s/_p/\%/g;
    $a = CGI::unescape($a);
    return $a;
}

sub cacheHidden {
    my ($self, $attr, $value) = (@_);
    my $ret = $self->hidden($attr, $value);
    push (@{$self->{CACHE}{HIDDEN}}, $attr.'='.$self->escape($value));
    return $ret;
}

sub showCache {
    my ($self) = @_;
    my @ret;
    foreach my $key (keys %{$self->{CACHE}}) {
	push (@ret, join ('&', @{$self->{CACHE}{$key}}));
    }
    return join ('&', @ret);
}

sub hidden1 {
    my ($self, $name, @values) = @_;
    $self->param($name, @values);
    return $self->SUPER::hidden($name, @values);
}

#'hidden' => <<'END_OF_FUNC',
sub hidden2 {
    my($self,@p) = self_or_default1(@_);

    # this is the one place where we departed from our standard
    # calling scheme, so we have to special-case (darn)
    my(@result,@value);
    my($name,$default,$override,@other) = 
	$self->rearrange([NAME,[DEFAULT,VALUE,VALUES],[OVERRIDE,FORCE]],@p);

    my $do_override = 0;
    if ( ref($p[0]) || substr($p[0],0,1) eq '-' || $self->use_named_parameters ) {
	@value = ref($default) ? @{$default} : $default;
	$do_override = $override;
    } else {
	foreach ($default,$override,@other) {
	    push(@value,$_) if defined($_);
	}
    }

    # use previous values if override is not set
    my @prev = $self->param($name);
    @value = @prev if !$do_override && @prev;

    $name=$self->escapeHTML($name);
    foreach (@value) {
	$_=$self->escapeHTML($_);
	push(@result,qq/<INPUT TYPE="hidden" NAME="$name" VALUE="$_">/);
    }
    return wantarray ? @result : join('',@result);
}
#END_OF_FUNC

sub self_or_default1 {
    return @_ if defined($_[0]) && (!ref($_[0])) &&($_[0] eq 'CGI');
    unless (defined($_[0]) && 
	    (ref($_[0]) eq 'CGI' || UNIVERSAL::isa($_[0],'CGI')) # slightly optimized for common case
	    ) {
	$Q = $CGI::DefaultClass->new unless defined($Q);
	unshift(@_,$Q);
    }
    return @_;
}

'submit1' => <<'END_OF_FUNC',
sub submit {
    my($self,@p) = self_or_default1(@_);

    my($label,$value,@other) = $self->rearrange([NAME,[VALUE,LABEL]],@p);

    $label=$self->escapeHTML($label);
    $value=$self->escapeHTML($value);

    my($name) = ' NAME=".submit"';
    $name = qq/ NAME="$label"/ if defined($label);
    $value = defined($value) ? $value : $label;
    my($val) = '';
    $val = qq/ VALUE="$value"/ if defined($value);
    my($other) = @other ? " @other" : '';
    return qq/<INPUT TYPE="submit"$name$val$other>/;
}
END_OF_FUNC

1;

__END__

=head1 NAME

W3C::Util::W3CDebugCGI - CGI wrapper to help debug CGIs

=head1 SYNOPSIS

  use W3C::Util::W3CDebugCGI;
  $query = new W3C::Util::W3CDebugCGI($0, $ARGV[0] eq 'DEBUG');
  # and call the script with a 'DEBUG' argument


=head1 DESCRIPTION

CGI wrapper to help debug CGIs.

This module is part of the W3C::Utils CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

perl(1).

=cut

